include \masm32\include\masm32rt.inc
    
.DATA
    szCharacterFile db "Character List.txt", 0      ; Name of file that stores character list
    szErrOpen       db "Error Opening File", 0      ; String used for debugging
    szErrRead       db "Error Reading File", 0      ; String used for debugging
    szSuccess       db "Success", 0                 ; String used for debugging
    szNewLine       db 13, 10                       ; Newline feed
    szClassName     db "lolcounter", 0
    szAppName       db "lolcounter", 0
    
.DATA?
; avoid globals
;    hMem dd ?                                       ; Pointer to the character list in memory
;    hArray dd ?                                     ; Pointer to the index array of characters
;    hArraySize dd ?                                 ; Number of indexes in the array
    hInstance HINSTANCE ?
    

; At least, a declaration should make clear what datatypes the parameters have. (IMO)
; The parameters names are an other story ;-)
ReadCharacterFile Proto ppString: ptr PCHAR          ; return: file size
ProcessString Proto pString:ptr CHAR                 ; return: number of strings
CreateArray Proto pStrings:ptr CHAR, nStrings:DWORD  ; 
PrintArray Proto ppStrings:ptr PCHAR                 ; 

.CODE

    WinMain proc hInst:HINSTANCE,hPrevInst:HINSTANCE,CmdLine:LPSTR,CmdShow:DWORD
        LOCAL wc:WNDCLASSEX                                            ; create local variables on stack
        LOCAL msg:MSG
        LOCAL hwnd:HWND
    
        mov   wc.cbSize,SIZEOF WNDCLASSEX                   ; fill values in members of wc
        mov   wc.style, CS_HREDRAW or CS_VREDRAW
        mov   wc.lpfnWndProc, OFFSET WndProc
        mov   wc.cbClsExtra,NULL
        mov   wc.cbWndExtra,NULL
        push  hInstance
        pop   wc.hInstance
        mov   wc.hbrBackground,COLOR_WINDOW+1
        mov   wc.lpszMenuName,NULL
        mov   wc.lpszClassName,OFFSET szClassName
        invoke LoadIcon,NULL,IDI_APPLICATION
        mov   wc.hIcon,eax
        mov   wc.hIconSm,eax
        invoke LoadCursor,NULL,IDC_ARROW
        mov   wc.hCursor,eax
        invoke RegisterClassEx, addr wc                       ; register our window class
        invoke CreateWindowEx,NULL,\
                    ADDR szClassName,\
                    ADDR szAppName,\
                    WS_OVERLAPPEDWINDOW,\
                    CW_USEDEFAULT,\
                    CW_USEDEFAULT,\
                    CW_USEDEFAULT,\
                    CW_USEDEFAULT,\
                    NULL,\
                    NULL,\
                    hInst,\
                    NULL
        mov   hwnd,eax
        invoke ShowWindow, hwnd,CmdShow               ; display our window on desktop
        invoke UpdateWindow, hwnd                                 ; refresh the client area
    
        .WHILE TRUE                                                         ; Enter message loop
                    invoke GetMessage, ADDR msg,NULL,0,0
                    .BREAK .IF (!eax)
                    invoke TranslateMessage, ADDR msg
                    invoke DispatchMessage, ADDR msg
       .ENDW
        mov     eax,msg.wParam                                            ; return exit code in eax
        ret
    WinMain endp
        
    WndProc proc hWnd:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM
        LOCAL fileSize:DWORD
        LOCAL nStrings:DWORD
        LOCAL pStrings:PCHAR
        LOCAL ppStrings:ptr PCHAR
        
        .IF uMsg == WM_CLOSE
        	invoke PostQuitMessage,NULL             ; quit our application
        .ELSEIF uMsg == WM_CREATE
            invoke ReadCharacterFile,ADDR pStrings
            mov fileSize,eax
            
            invoke ProcessString, pStrings
            mov nStrings,eax
            
            invoke CreateArray, pStrings, nStrings
            mov ppStrings,eax
            
            invoke PrintArray, ppStrings
           
           	invoke GlobalFree, pStrings
            invoke GlobalFree, ppStrings
           
        .ELSE
            invoke DefWindowProc,hWnd,uMsg,wParam,lParam     ; Default message processing
            ret
        .ENDIF
        xor eax,eax
        ret
    WndProc endp 

    ReadCharacterFile proc ppString: ptr PCHAR        ; Start of ReadCharacterFile function
		LOCAL hMem:HANDLE
        LOCAL hFile:HANDLE                          ; Handle to the file being opened
        LOCAL fileSize:DWORD                        ; Stores the size of the file
        LOCAL bytesRead:DWORD                       ; Stores how many bytes have been read
        
        mov fileSize,0
        mov eax,ppString
        mov HANDLE ptr [eax],0
        
        invoke CreateFile, ADDR szCharacterFile,    ; Creates a file handle pointing to the character file
                           GENERIC_READ, NULL, 
                           NULL, OPEN_EXISTING, 
                           FILE_ATTRIBUTE_NORMAL, 
                           NULL
        or eax, eax                                 ; Sets eax to itself
        jz OpenFail                                 ; If eax == 0, jump to OpenFail
        mov hFile, eax                              ; Store the file pointer in hFile

        invoke GetFileSize, eax, 0                  ; Get the size of the file
        mov fileSize, eax                           ; Store the file size in fileSize
        inc eax                                     ; Add 1 to eax (extra byte for null terminator)

        invoke GlobalAlloc, GMEM_FIXED, eax         ; Allocate enough memory to store the file
        mov hMem, eax                               ; Put the pointer to that memory in hMem
        mov edx,ppString
        mov PCHAR ptr [edx],eax
        add eax, fileSize                           ; Make eax point to the end of the buffer
        mov BYTE PTR [eax], 0                       ; Set last byte to 0 (Null terminator)
        
        invoke ReadFile, hFile, hMem,               ; Reads the file into the hMem buffer
               fileSize, ADDR bytesRead, NULL    
        or eax, eax                                 ; Set eax to itself
        jz ReadFail                                 ; If eax == 0, jump to ReadFail
        
        ReadFail:
            invoke CloseHandle, hFile               ; Closes the file handle

        OpenFail:
        	mov eax,fileSize						; return size
            ret                                     ; Return from function
               
    ReadCharacterFile endp                          ; End of ReadCharacterFile function

    ProcessString proc USES esi pString:ptr CHAR
        mov esi, pString
        xor ecx, ecx
        CheckByte:
            mov al, [esi]
            inc esi
            cmp BYTE PTR al, ';'
            jz ReplaceByte
            cmp BYTE PTR al, 0
            jnz CheckByte
            inc ecx
            
            mov eax,ecx  ; return number of strings
            ret

            ReplaceByte:
                mov BYTE PTR [esi - 1], 0
                inc ecx
                jmp CheckByte
                
    ProcessString endp

	; UESE: save edi and esi to be in consent with the Win/IntelABI
    CreateArray proc USES esi edi pStrings:ptr CHAR, nStrings:DWORD
        mov eax, nStrings

        inc eax
        lea eax,[eax*4] ; zero terminated pointer array!
		
		; global alloc clears the memory for us
        invoke GlobalAlloc, GMEM_FIXED or GMEM_ZEROINIT, eax
		mov edi,eax
        
        ;mov ecx, dwSize
        ;mov DWORD PTR [eax+ecx*4], 0
       
        mov esi, pStrings
        mov ecx, nStrings
        xor edx,edx     ; first element in pointer array
        
        CheckBytes:

            cmp ecx, 0
            jz Done
            mov [edi + edx], esi ; save pointer to current string
            
            ; scan string for zero
        @@:
            movzx eax,BYTE ptr [esi]
            inc esi
            test eax,eax
            jnz @B
        @@:
            
            add edx, 4
            dec ecx
            jmp CheckBytes
            
            Done:
            	mov eax,edi ; return ppStrings
                ret
                
    CreateArray endp

    PrintArray proc uses esi edi ppStrings:ptr PCHAR 
        mov esi, ppStrings
        mov ecx, 0
        
        CheckBytes:
            mov edi, [esi + ecx * 4]
            cmp edi, 0
            jz Done
            push ecx
            invoke MessageBox, NULL, [esi + ecx * 4], offset szSuccess, MB_OK
            pop ecx
            inc ecx
            jmp CheckBytes
            
            Done:
                ret
                
    PrintArray endp
    
    start:
        invoke GetModuleHandle, NULL
        mov hInstance,eax
        invoke WinMain, hInstance, NULL, NULL, SW_SHOWDEFAULT
        invoke ExitProcess, eax                           
    end start